@@ -30,8 +30,7 @@
 #endif
 
 #include <gtk/gtk.h>
-
-#include <dbus/dbus-glib.h>
+#include <gtk/gtkx.h>
 
 #include <xfconf/xfconf.h>
 #include <libxfce4ui/libxfce4ui.h>
@@ -55,7 +54,7 @@ xfce_notifyd_config_show_notification_preview(GtkWindow *parent_window)
 
     notification =
         notify_notification_new(_("Notification Preview"),
-                                _("This is how notifications will look like"),
+                                _("This is what notifications will look like"),
                                 "xfce4-notifyd");
 
     notify_notification_add_action(notification,
@@ -161,7 +160,7 @@ list_store_add_themes_in_dir(GtkListStore *ls,
             continue;
 
         filename =
-            g_build_filename(path, file, "xfce-notify-4.0", "gtkrc", NULL);
+            g_build_filename(path, file, "xfce-notify-4.0", "gtk.css", NULL);
 
         if(g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
             GtkTreeIter iter;
@@ -256,10 +255,10 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
 
     if(!xfconf_init(&error)) {
         xfce_message_dialog(NULL, _("Xfce Notify Daemon"),
-                            GTK_STOCK_DIALOG_ERROR,
+                            "dialog-error",
                             _("Settings daemon is unavailable"),
                             error->message,
-                            GTK_STOCK_QUIT, GTK_RESPONSE_ACCEPT,
+                            "application-exit", GTK_RESPONSE_ACCEPT,
                             NULL);
         exit(EXIT_FAILURE);
     }
@@ -305,7 +304,7 @@ main(int argc,
     GtkWidget *settings_dialog = NULL;
     GtkBuilder *builder;
     gboolean opt_version = FALSE;
-    GdkNativeWindow opt_socket_id = 0;
+    gint32 opt_socket_id = 0;
     GOptionEntry option_entries[] = {
         { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, N_("Display version information"), NULL },
         { "socket-id", 's', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_INT, &opt_socket_id, N_("Settings manager socket"), N_("SOCKET_ID") },
@@ -360,8 +359,11 @@ main(int argc,
                          G_CALLBACK(gtk_main_quit), NULL);
 
         plug_child = GTK_WIDGET(gtk_builder_get_object(builder, "plug-child"));
-        gtk_widget_reparent(plug_child, plug);
-        gtk_widget_show(plug_child);
+
+        /* In the glade file, plug_child has parent, so remove it first */
+        gtk_container_remove (GTK_CONTAINER(gtk_widget_get_parent(plug_child)), plug_child);
+        gtk_container_add (GTK_CONTAINER(plug), plug_child);
+        gtk_widget_set_valign (plug_child, GTK_ALIGN_START);
 
         gdk_notify_startup_complete();
         g_object_unref(G_OBJECT(builder));
